@@ -269,7 +269,7 @@ static int runmain(int argc, char *argv[]) {
                                     "  subject - subject name of proxy certificate.\n"
                                     "  identity - identity subject name of proxy certificate.\n"
                                     "  issuer - issuer subject name of proxy certificate.\n"
-                                    "  ca - subject name of CA ehich issued initial certificate\n"
+                                    "  ca - subject name of CA which issued initial certificate.\n"
                                     "  path - file system path to file containing proxy.\n"
                                     "  type - type of proxy certificate.\n"
                                     "  validityStart - timestamp when proxy validity starts.\n"
@@ -309,7 +309,7 @@ static int runmain(int argc, char *argv[]) {
                     istring("path"), proxy_path);
 
   std::string cert_path;
-  options.AddOption('C', "cert", istring("path to the certificate file, it can be either PEM, DER, or PKCS12 formated"),
+  options.AddOption('C', "cert", istring("path to the certificate file, it can be either PEM, DER, or PKCS12 formatted"),
                     istring("path"), cert_path);
 
   std::string key_path;
@@ -612,8 +612,11 @@ static int runmain(int argc, char *argv[]) {
                  "Please make sure this file exists.", proxy_path);
       return EXIT_FAILURE;
     }
-
     Arc::Credential holder(proxy_path, "", "", "");
+    if(!holder.GetCert()) {
+      logger.msg(Arc::ERROR, "Cannot process proxy file at %s.", proxy_path);
+      return EXIT_FAILURE;
+    }
     std::cout << Arc::IString("Subject: %s", holder.GetDN()) << std::endl;
     std::cout << Arc::IString("Issuer: %s", holder.GetIssuerName()) << std::endl;
     std::cout << Arc::IString("Identity: %s", holder.GetIdentityName()) << std::endl;
@@ -672,30 +675,38 @@ static int runmain(int argc, char *argv[]) {
         std::cout << "VO        : "<<voms_attributes[n].voname << std::endl;
         std::cout << "subject   : "<<voms_attributes[n].holder << std::endl;
         std::cout << "issuer    : "<<voms_attributes[n].issuer << std::endl;
-        bool found_uri = false;
         for(int i = 0; i < voms_attributes[n].attributes.size(); i++) {
           std::string attr = voms_attributes[n].attributes[i];
           std::string::size_type pos;
           if((pos = attr.find("hostname=")) != std::string::npos) {
-            if(found_uri) continue;
-            std::string str = attr.substr(pos+9);
-            std::string::size_type pos1 = str.find("/");
-            if(pos1 != std::string::npos) str = str.substr(0, pos1);
-            std::cout << "uri       : " << str <<std::endl;
-            found_uri = true;
+            std::list<std::string> attr_elements;
+            Arc::tokenize(attr, attr_elements, "/");
+            // remove /voname= prefix
+            if ( ! attr_elements.empty() ) attr_elements.pop_front();
+            if ( attr_elements.empty() ) {
+              logger.msg(Arc::WARNING, "Malformed VOMS AC attribute %s", attr);
+              continue;
+            }
+            // policyAuthority (URI) and AC tags
+            if ( attr_elements.size() == 1 ) {
+              std::string uri = attr_elements.front().substr(9);
+              std::cout << "uri       : " << uri <<std::endl;
+            } else {
+              std::string fqan = "";
+              attr_elements.pop_front();
+              // tags rendering is not defined in GFD.182
+              // tags are assigned to members, groups and roles
+              // FQAN-based ARC rendering is used
+              while (! attr_elements.empty () ) {
+                fqan.append("/").append(attr_elements.front());
+                attr_elements.pop_front();
+              }
+              std::cout << "tag       : " << fqan <<std::endl;
+            }
           }
           else {
-            if(attr.find("Role=") == std::string::npos)
-              std::cout << "attribute : " << attr <<"/Role=NULL/Capability=NULL" <<std::endl;
-            else if(attr.find("Capability=") == std::string::npos)
-              std::cout << "attribute : " << attr <<"/Capability=NULL" <<std::endl;
-            else
-              std::cout << "attribute : " << attr <<std::endl;
-          }
-
-          if((pos = attr.find("Role=")) != std::string::npos) {
-            std::string str = attr.substr(pos+5);
-            std::cout << "attribute : role = " << str << " (" << voms_attributes[n].voname << ")"<<std::endl;
+            // Short FQANs are GFD.182 compliant, no need to add Role=NULL
+            std::cout << "attribute : " << attr <<std::endl;
           }
 
           //std::cout << "attribute : "<<voms_attributes[n].attributes[i]<<std::endl;
@@ -723,11 +734,27 @@ static int runmain(int argc, char *argv[]) {
     return EXIT_SUCCESS;
   }
   if(infoitemlist.size() > 0) {
-    std::vector<Arc::VOMSACInfo> voms_attributes;
+    if (proxy_path.empty()) {
+      logger.msg(Arc::ERROR, "Cannot find the path of the proxy file, "
+                 "please setup environment X509_USER_PROXY, "
+                 "or proxypath in a configuration file");
+      return EXIT_FAILURE;
+    }
+    else if (!(Glib::file_test(proxy_path, Glib::FILE_TEST_EXISTS))) {
+      logger.msg(Arc::ERROR, "Cannot find file at %s for getting the proxy. "
+                 "Please make sure this file exists.", proxy_path);
+      return EXIT_FAILURE;
+    }
     Arc::Credential holder(proxy_path, "", "", "");
+    if(!holder.GetCert()) {
+      logger.msg(Arc::ERROR, "Cannot process proxy file at %s.", proxy_path);
+      return EXIT_FAILURE;
+    }
     Arc::VOMSTrustList voms_trust_dn;
     voms_trust_dn.AddRegex(".*");
+    std::vector<Arc::VOMSACInfo> voms_attributes;
     parseVOMSAC(holder, ca_dir, "", voms_dir, voms_trust_dn, voms_attributes, true, true);
+    bool unknownInfo = false;
     for(std::list<std::string>::iterator ii = infoitemlist.begin();
                            ii != infoitemlist.end(); ++ii) {
       if(*ii == "subject") {
@@ -800,8 +827,11 @@ static int runmain(int argc, char *argv[]) {
         std::cout << signTypeToString(holder.GetSigningAlgorithm()) << std::endl;
       } else {
         logger.msg(Arc::ERROR, "Information item '%s' is not known",*ii);
+        unknownInfo = true;
       }
     }
+    if (unknownInfo)
+      return EXIT_FAILURE;
     return EXIT_SUCCESS;
   }
 
@@ -955,7 +985,7 @@ static int runmain(int argc, char *argv[]) {
     } else {
       // otherwise start - optionally - and end are set, period is derived
       if(validityEnd < validityStart) {
-        std::cerr << Arc::IString("The end time that you set: %s is before start time:%s.", (std::string)validityEnd,(std::string)validityStart) << std::endl;
+        std::cerr << Arc::IString("The end time that you set: %s is before start time: %s.", (std::string)validityEnd,(std::string)validityStart) << std::endl;
         // error
         return EXIT_FAILURE;
       }
