@@ -330,7 +330,7 @@ SecHandlerStatus ArgusPEPClient::Handle(Arc::Message* msg) const {
                 secattr.GetXML(xml);
                 logger.msg(Arc::INFO,"Not authorized according to request:\n%s",xml);
             } else if(conversion == conversion_subject) {
-                logger.msg(Arc::INFO,"%s is not authorized to do action %s in resource %s ",
+                logger.msg(Arc::INFO,"%s is not authorized to do action %s in resource %s",
                            subject, action, resource);
             } else {
                 logger.msg(Arc::INFO,"Not authorized");
@@ -358,12 +358,12 @@ SecHandlerStatus ArgusPEPClient::Handle(Arc::Message* msg) const {
 int ArgusPEPClient::create_xacml_request(xacml_request_t ** request,const char * subjectid, const char * resourceid, const char * actionid) const {
     xacml_subject_t * subject= xacml_subject_create();
     if (subject == NULL) {
-        logger.msg(Arc::DEBUG, "Subject of request is null \n");
+        logger.msg(Arc::DEBUG, "Subject of request is null");
         return 1;
     }
     xacml_attribute_t * subject_attr_id= xacml_attribute_create(XACML_SUBJECT_ID);
     if (subject_attr_id == NULL) {
-    logger.msg(Arc::DEBUG,"Can not create XACML SubjectAttribute: %s\n", XACML_SUBJECT_ID);
+    logger.msg(Arc::DEBUG,"Can not create XACML SubjectAttribute: %s", XACML_SUBJECT_ID);
         xacml_subject_delete(subject);
         return 1;
     }
@@ -372,13 +372,13 @@ int ArgusPEPClient::create_xacml_request(xacml_request_t ** request,const char *
     xacml_subject_addattribute(subject,subject_attr_id); 
     xacml_resource_t * resource = xacml_resource_create();
     if (resource == NULL) {
-        logger.msg(Arc::DEBUG, "Can not create XACML Resource \n");
+        logger.msg(Arc::DEBUG, "Can not create XACML Resource");
         xacml_subject_delete(subject);
         return 2;
     }
     xacml_attribute_t * resource_attr_id = xacml_attribute_create(XACML_RESOURCE_ID);
     if (resource_attr_id == NULL) {
-        logger.msg(Arc::DEBUG,"Can not create XACML ResourceAttribute: %s\n", XACML_RESOURCE_ID);
+        logger.msg(Arc::DEBUG,"Can not create XACML ResourceAttribute: %s", XACML_RESOURCE_ID);
         xacml_subject_delete(subject);
         xacml_resource_delete(resource);
         return 2;
@@ -387,14 +387,14 @@ int ArgusPEPClient::create_xacml_request(xacml_request_t ** request,const char *
     xacml_resource_addattribute(resource,resource_attr_id);
     xacml_action_t * action= xacml_action_create();
     if (action == NULL) {
-        logger.msg(Arc::DEBUG,"Can not create XACML Action\n");
+        logger.msg(Arc::DEBUG,"Can not create XACML Action");
         xacml_subject_delete(subject);
         xacml_resource_delete(resource);
         return 3;
     }
     xacml_attribute_t * action_attr_id= xacml_attribute_create(XACML_ACTION_ID);
     if (action_attr_id == NULL) {
-        logger.msg(Arc::DEBUG,"Can not create XACML ActionAttribute: %s\n", XACML_ACTION_ID);
+        logger.msg(Arc::DEBUG,"Can not create XACML ActionAttribute: %s", XACML_ACTION_ID);
         xacml_subject_delete(subject);
         xacml_resource_delete(resource);
         xacml_action_delete(action);
@@ -404,7 +404,7 @@ int ArgusPEPClient::create_xacml_request(xacml_request_t ** request,const char *
     xacml_action_addattribute(action,action_attr_id);
     *request= xacml_request_create();
     if (*request == NULL) {
-    logger.msg(Arc::DEBUG,"Can not create XACML request\n");
+    logger.msg(Arc::DEBUG,"Can not create XACML request");
     xacml_subject_delete(subject);
         xacml_resource_delete(resource);
         xacml_action_delete(action);
@@ -757,7 +757,7 @@ logger.msg(Arc::DEBUG,"Doing CREAM request");
     std::string endpoint = get_resource(auths,attrs);
     if(endpoint.empty()) throw ierror("Failed to extract resource identifier");
     xacml_attribute_addvalue(attr, endpoint.c_str());
-    logger.msg(Arc::DEBUG,"Adding resoure-id value: %s",endpoint);
+    logger.msg(Arc::DEBUG,"Adding resource-id value: %s",endpoint);
     xacml_attribute_setdatatype(attr, XACML_DATATYPE_STRING);
     xacml_resource_addattribute(resource,attr); attr = NULL;
 
