@@ -6,20 +6,19 @@
 
 #include <cstdlib>
 #include <iostream>
+#include <arc/IString.h>
 
 #ifdef HAVE_GLIBMM_OPTIONCONTEXT_SET_SUMMARY
 #include <glibmm/optioncontext.h>
-#ifndef HAVE_GLIBMM_OPTIONCONTEXT_GET_HELP
-#include <arc/IString.h>
-#endif
 #else
 #include <getopt.h>
-#include <arc/IString.h>
 #include <arc/StringConv.h>
 #endif
 
 #include "OptionParser.h"
 
+#define ARC_OPTION_DEFAULT_GROUP "default"
+
 namespace Arc {
 
 #ifdef HAVE_GLIBMM_OPTIONCONTEXT_SET_SUMMARY
@@ -183,17 +182,19 @@ namespace Arc {
       description(description) {}
 
   OptionParser::~OptionParser() {
-    for (std::list<OptionBase*>::iterator it = options.begin();
-         it != options.end(); it++)
-      delete *it;
+    std::list<std::pair<OptionBase*, std::string> >::iterator it = options.begin();
+    while ( it != options.end() ) {
+        delete it->first;
+        it->first = NULL;
+        it = options.erase(it);
+    }
   }
 
   void OptionParser::AddOption(const char shortOpt,
                                const std::string& longOpt,
                                const std::string& optDesc,
                                bool& value) {
-    options.push_back(new BoolOption(shortOpt, longOpt,
-                                     optDesc, "", value));
+    GroupAddOption(ARC_OPTION_DEFAULT_GROUP, shortOpt, longOpt, optDesc, value);
   }
 
   void OptionParser::AddOption(const char shortOpt,
@@ -201,8 +202,7 @@ namespace Arc {
                                const std::string& optDesc,
                                const std::string& argDesc,
                                int& value) {
-    options.push_back(new IntOption(shortOpt, longOpt,
-                                    optDesc, argDesc, value));
+    GroupAddOption(ARC_OPTION_DEFAULT_GROUP, shortOpt, longOpt, optDesc, argDesc, value);
   }
 
   void OptionParser::AddOption(const char shortOpt,
@@ -210,8 +210,7 @@ namespace Arc {
                                const std::string& optDesc,
                                const std::string& argDesc,
                                std::string& value) {
-    options.push_back(new StringOption(shortOpt, longOpt,
-                                       optDesc, argDesc, value));
+    GroupAddOption(ARC_OPTION_DEFAULT_GROUP, shortOpt, longOpt, optDesc, argDesc, value);
   }
 
   void OptionParser::AddOption(const char shortOpt,
@@ -219,10 +218,57 @@ namespace Arc {
                                const std::string& optDesc,
                                const std::string& argDesc,
                                std::list<std::string>& value) {
-    options.push_back(new StringListOption(shortOpt, longOpt,
-                                           optDesc, argDesc, value));
+    GroupAddOption(ARC_OPTION_DEFAULT_GROUP, shortOpt, longOpt, optDesc, argDesc, value);
   }
 
+  void OptionParser::GroupAddOption(const std::string groupID,
+                               const char shortOpt,
+                               const std::string& longOpt,
+                               const std::string& optDesc,
+                               bool& value) {
+    optgroups[groupID];
+    options.push_back(std::pair<OptionBase*, std::string>
+      (new BoolOption(shortOpt, longOpt, optDesc, "", value), groupID));
+  }
+
+  void OptionParser::GroupAddOption(const std::string groupID,
+                               const char shortOpt,
+                               const std::string& longOpt,
+                               const std::string& optDesc,
+                               const std::string& argDesc,
+                               int& value) {
+    optgroups[groupID];
+    options.push_back(std::pair<OptionBase*, std::string>
+      (new IntOption(shortOpt, longOpt,optDesc, argDesc, value), groupID));
+  }
+
+  void OptionParser::GroupAddOption(const std::string groupID,
+                               const char shortOpt,
+                               const std::string& longOpt,
+                               const std::string& optDesc,
+                               const std::string& argDesc,
+                               std::string& value) {
+    optgroups[groupID];
+    options.push_back(std::pair<OptionBase*, std::string>
+      (new StringOption(shortOpt, longOpt, optDesc, argDesc, value), groupID));
+  }
+
+  void OptionParser::GroupAddOption(const std::string groupID,
+                               const char shortOpt,
+                               const std::string& longOpt,
+                               const std::string& optDesc,
+                               const std::string& argDesc,
+                               std::list<std::string>& value) {
+    optgroups[groupID];
+    options.push_back(std::pair<OptionBase*, std::string>
+      (new StringListOption(shortOpt, longOpt,optDesc, argDesc, value), groupID));
+  }
+
+  void OptionParser::DefineOptionsGroup(const std::string groupID, const std::string& groupName) {
+    optgroups[groupID] = groupName;
+  }
+
+
 #ifdef HAVE_GLIBMM_OPTIONCONTEXT_SET_SUMMARY
   std::list<std::string> OptionParser::Parse(int argc, char **argv) {
 
@@ -239,39 +285,75 @@ namespace Arc {
     if (!description.empty())
       ctx.set_description(description);
     ctx.set_translation_domain(PACKAGE);
-    Glib::OptionGroup grp("main", "Main Group");
-    grp.set_translation_domain(PACKAGE);
+
+#ifdef HAVE_GLIBMM_OPTIONCONTEXT_GET_HELP
+    // if get_help() is supported disable native -? --help processing 
+    // etc to show --help-all by default
+    ctx.set_help_enabled(false);
+#endif
 
     bool h_value = false;
     BoolOption h_entry('h', "help", "Show help options", "", h_value);
-    h_entry.AddEntry(grp);
 
-    for (std::list<OptionBase*>::iterator it = options.begin();
-         it != options.end(); it++)
-      (*it)->AddEntry(grp);
+    for (std::map<std::string, std::string>::iterator g_it = optgroups.begin();
+         g_it != optgroups.end(); g_it++) {
 
-    ctx.set_main_group(grp);
+      Glib::OptionGroup *grp;
+
+      if ( g_it->first == ARC_OPTION_DEFAULT_GROUP ) {
+          grp = new Glib::OptionGroup(ARC_OPTION_DEFAULT_GROUP, "Main Group");
+          grp->set_translation_domain(PACKAGE);
+          h_entry.AddEntry(*grp);
+      } else {
+          std::string grp_name;
+          if (g_it->second.empty()) {
+             grp_name = IString("Options Group %s:", g_it->first).str();
+          } else {
+             grp_name = IString("%s:", g_it->second).str();
+          }
+          grp = new Glib::OptionGroup(g_it->first, grp_name, IString("Show %s help options", g_it->first).str());
+      }
+
+      bool options_in_group = false;
+      for (std::list<std::pair<OptionBase*, std::string> >::iterator it = options.begin();
+           it != options.end(); it++) {
+        if ( (*it).second == g_it->first ) {
+          options_in_group = true;
+          (*it).first->AddEntry(*grp);
+        }
+      }
+
+      if ( g_it->first == ARC_OPTION_DEFAULT_GROUP ) {
+        ctx.set_main_group(*grp);
+      } else {
+        if ( options_in_group ) ctx.add_group(*grp);
+      }
+    }
 
     try {
       ctx.parse(argc, argv);
-    } catch (Glib::OptionError& err) {
-      std::cout << err.what() << std::endl;
+    } catch (const Glib::OptionError& err) {
+      try {
+        std::cerr << err.what() << std::endl;
+      } catch (const Glib::Exception& err2) {
+        std::cerr << IString("Failed to parse comamnd line options") << std::endl;
+      }
       exit(1);
     }
     if(h_value) {
 #ifdef HAVE_GLIBMM_OPTIONCONTEXT_GET_HELP
       // In some versions Glib's get_help returns text which fails in Glib's internal translator of operator<<.
       // Hence using internal string without additional translation.
-      std::cout << ctx.get_help().c_str() << std::endl;
+      std::cout << ctx.get_help(false).c_str() << std::endl;
 #else
       std::cout << IString("Use -? to get usage description") << std::endl;
 #endif
       exit(0);
     }
 
-    for (std::list<OptionBase*>::iterator it = options.begin();
+    for (std::list<std::pair<OptionBase*, std::string> >::iterator it = options.begin();
          it != options.end(); it++)
-      (*it)->Result();
+      (*it).first->Result();
 
     std::list<std::string> params;
     for (int i = 1; i < argc; i++)
@@ -302,14 +384,14 @@ namespace Arc {
     struct option *longoptions = new struct option[options.size() + 3];
     int i = 0;
     std::string optstring;
-    for (std::list<OptionBase*>::iterator it = options.begin();
+    for (std::list<std::pair<OptionBase*,std::string> >::iterator it = options.begin();
          it != options.end(); it++) {
-      setopt(longoptions[i], (*it)->longOpt.c_str(),
-             (*it)->argDesc.empty() ? no_argument : required_argument,
-             NULL, (*it)->shortOpt?(*it)->shortOpt:(i+0x100));
-      if((*it)->shortOpt) {
-        optstring += (*it)->shortOpt;
-        if (!(*it)->argDesc.empty()) optstring += ':';
+      setopt(longoptions[i], (*it).first->longOpt.c_str(),
+             (*it).first->argDesc.empty() ? no_argument : required_argument,
+             NULL, (*it).first->shortOpt?(*it).first->shortOpt:(i+0x100));
+      if((*it).first->shortOpt) {
+        optstring += (*it).first->shortOpt;
+        if (!(*it).first->argDesc.empty()) optstring += ':';
       }
       ++i;
     }
@@ -353,19 +435,39 @@ namespace Arc {
         std::cout << IString("Help Options:") << std::endl;
         std::cout << "  -h, -?, --help    " << IString("Show help options")
                   << std::endl << std::endl;
-        std::cout << IString("Application Options:") << std::endl;
-        for (std::list<OptionBase*>::iterator it = options.begin();
-             it != options.end(); it++) {
-          std::cout << "  ";
-          if ((*it)->shortOpt) {
-            std::cout << "-" << (*it)->shortOpt << ", ";
+        for (std::map<std::string, std::string>::iterator g_it = optgroups.begin();
+             g_it != optgroups.end(); g_it++) {
+          std::string group_header = "Application Options:";
+          if (g_it->second.empty()) {
+            if ( g_it->first != ARC_OPTION_DEFAULT_GROUP ) {
+              group_header = IString("Options Group %s:", g_it->first).str();
+            }
+          } else {
+            group_header = IString("%s:", g_it->second).str();
+          }
+          bool print_header = true;
+          for (std::list<std::pair<OptionBase*,std::string> >::iterator it = options.begin();
+               it != options.end(); it++) {
+            // skip options in other groups
+            if ( (*it).second != g_it->first) continue;
+            // print header only if at least one option in this group
+            if (print_header) {
+              print_header = false;
+              std::cout << group_header << std::endl;
+            }
+            // print option description
+            std::cout << "  ";
+            OptionBase* ob = (*it).first;
+            if (ob->shortOpt) {
+              std::cout << "-" << ob->shortOpt << ", ";
+            }
+            std::cout << "--";
+            if (ob->argDesc.empty())
+              std::cout << std::setw(20+4*(ob->shortOpt == 0)) << std::left << ob->longOpt;
+            else
+              std::cout << ob->longOpt << "=" << std::setw(19-(ob->longOpt).length()+((ob->shortOpt == 0))*4) << std::left << IString(ob->argDesc);
+            std::cout << "  " << IString(ob->optDesc) << std::endl;
           }
-          std::cout << "--";
-          if ((*it)->argDesc.empty())
-            std::cout << std::setw(20+4*((*it)->shortOpt == 0)) << std::left << (*it)->longOpt;
-          else
-            std::cout << (*it)->longOpt << "=" << std::setw(19-((*it)->longOpt).length()+(((*it)->shortOpt == 0))*4) << std::left << IString((*it)->argDesc);
-          std::cout << "  " << IString((*it)->optDesc) << std::endl;
         }
         std::cout << std::endl;
         if (!description.empty())
@@ -374,12 +476,12 @@ namespace Arc {
         exit(0);
       }
       i = 0;
-      for (std::list<OptionBase*>::iterator it = options.begin();
+      for (std::list<std::pair<OptionBase*,std::string> >::iterator it = options.begin();
            it != options.end(); it++) {
-        int o = (*it)->shortOpt;
+        int o = (*it).first->shortOpt;
         if(!o) o = longoptions[i].val;
         if (opt == o) {
-          if (!(*it)->Set(optarg ? optarg : "")) {
+          if (!(*it).first->Set(optarg ? optarg : "")) {
             delete[] longoptions;
             exit(1);
           }
