@@ -769,25 +769,46 @@ namespace Arc {
         }
       }
       source_url.AddCheckSumObject(&crc_source);
-      if (source_url.SupportsTransfer()) {
-        logger.msg(INFO, "Using internal transfer method of %s", source_url.str());
-        URL dest_url(cacheable ? chdest.GetURL() : destination.GetURL());
-        DataStatus datares = source_url.Transfer(dest_url, true, show_progress ? transfer_cb : NULL);
-        if (!datares.Passed()) {
-          DataHandle d(destination.GetURL(), destination.GetUserConfig());
-          Delete(*d);
-          if (source.NextLocation()) {
-            logger.msg(VERBOSE, "(Re)Trying next source");
-            continue;
+      bool try_another_transfer = true;
+      if (try_another_transfer) {
+        if (source_url.SupportsTransfer()) {
+          logger.msg(INFO, "Using internal transfer method of %s", source_url.str());
+          URL dest_url(cacheable ? chdest.GetURL() : destination.GetURL());
+          DataStatus datares = source_url.Transfer(dest_url, true, show_progress ? transfer_cb : NULL);
+          if (!datares.Passed()) {
+            if (source.NextLocation()) {
+              logger.msg(VERBOSE, "(Re)Trying next source");
+              continue;
+            }
+            if (cacheable)
+              cache.StopAndDelete(canonic_url);
+            // Check if SupportsTransfer was too optimistic
+            if (datares != DataStatus::UnimplementedError) return datares;
+              logger.msg(INFO, "Internal transfer method is not supported for %s", source_url.str());
+          } else {
+            try_another_transfer = false;
           }
-          if (cacheable)
-            cache.StopAndDelete(canonic_url);
-          return datares;
         }
-      } else if (destination.SupportsTransfer()) {
-        logger.msg(INFO, "Using internal transfer method of %s", destination.str());
-        return destination.Transfer(source_url.GetURL(), false, transfer_cb);
-      } else {
+      }
+      if (try_another_transfer) {
+        if (destination.SupportsTransfer()) {
+          logger.msg(INFO, "Using internal transfer method of %s", destination.str());
+          DataStatus datares = destination.Transfer(source_url.GetURL(), false, show_progress ? transfer_cb : NULL);
+          if (!datares.Passed()) {
+            if (source.NextLocation()) {
+              logger.msg(VERBOSE, "(Re)Trying next source");
+              continue;
+            }
+            // Check if SupportsTransfer was too optimistic
+            if (datares != DataStatus::UnimplementedError) return datares;
+            logger.msg(INFO, "Internal transfer method is not supported for %s", destination.str());
+          } else {
+            try_another_transfer = false;
+          }
+        }
+      }
+      if (try_another_transfer) {
+        logger.msg(INFO, "Using buffered transfer method");
         unsigned int wait_time;
         DataStatus datares = source_url.PrepareReading(max_inactivity_time, wait_time);
         if (!datares.Passed()) {
