@@ -22,6 +22,7 @@
 #include <arc/StringConv.h>
 #include <arc/FileUtils.h>
 #include <arc/Utils.h>
+#include <arc/otokens/openid_metadata.h>
 #include "job.h"
 #include "grid-manager/log/JobLog.h"
 #include "grid-manager/log/JobsMetrics.h"
@@ -237,6 +238,43 @@ std::string ARexSecAttr::get(const std::string& id) const {
   return "";
 };
 
+static bool match_lists(const std::list<std::pair<bool,std::string> >& list1, const std::list<std::string>& list2, std::string& matched) {
+  for(std::list<std::pair<bool,std::string> >::const_iterator l1 = list1.begin(); l1 != list1.end(); ++l1) {
+    for(std::list<std::string>::const_iterator l2 = list2.begin(); l2 != list2.end(); ++l2) {
+      if((l1->second) == (*l2)) {
+        matched = l1->second;
+        return l1->first;
+      };
+    };
+  };
+  return false;
+}
+
+static bool match_groups(std::list<std::pair<bool,std::string> > const & groups, Arc::Message& inmsg) {
+  std::string matched_group;
+  if(!groups.empty()) {
+    Arc::MessageAuth* auth = inmsg.Auth();
+    if(auth) {
+      Arc::SecAttr* sattr = auth->get("ARCLEGACY");
+      if(sattr) {
+        if(match_lists(groups, sattr->getAll("GROUP"), matched_group)) {
+          return true;
+        };
+      };
+    };
+    auth = inmsg.AuthContext();
+    if(auth) {
+      Arc::SecAttr* sattr = auth->get("ARCLEGACY");
+      if(sattr) {
+        if(match_lists(groups, sattr->getAll("GROUP"), matched_group)) {
+          return true;
+        };
+      };
+    };
+  };
+  return false;
+}
+
 static Arc::XMLNode ESCreateResponse(Arc::PayloadSOAP& res,const char* opname) {
   Arc::XMLNode response = res.NewChild(ES_CREATE_NPREFIX + ":" + opname + "Response");
   return response;
@@ -397,16 +435,17 @@ Arc::MCC_Status ARexService::make_empty_response(Arc::Message& outmsg) {
   return Arc::MCC_Status(Arc::STATUS_OK);
 }
 
-ARexConfigContext* ARexService::get_configuration(Arc::Message& inmsg) {
-  ARexConfigContext* config = NULL;
+ARexGMConfig* ARexService::get_configuration(Arc::Message& inmsg) {
   Arc::MessageContextElement* mcontext = (*inmsg.Context())["arex.gmconfig"];
   if(mcontext) {
     try {
-      config = dynamic_cast<ARexConfigContext*>(mcontext);
-      logger_.msg(Arc::DEBUG,"Using cached local account '%s'", config->User().Name());
+      ARexConfigContext* config = dynamic_cast<ARexConfigContext*>(mcontext);
+      if(config) {
+        logger_.msg(Arc::DEBUG,"Using cached local account '%s'", config->User().Name());
+        return config;
+      }
     } catch(std::exception& e) { };
   };
-  if(config) return config;
   // TODO: do configuration detection
   // TODO: do mapping to local unix name
   std::string uname;
@@ -432,6 +471,46 @@ ARexConfigContext* ARexService::get_configuration(Arc::Message& inmsg) {
   };
   logger_.msg(Arc::DEBUG,"Using local account '%s'",uname);
   std::string grid_name = inmsg.Attributes()->get("TLS:IDENTITYDN");
+  if(grid_name.empty()) {
+    // Try tokens if TLS has no information about user identity
+    logger_.msg(Arc::ERROR, "TLS provides no identity, going for OTokens");
+    grid_name = inmsg.Attributes()->get("OTOKENS:IDENTITYDN");
+    /*
+    Below is an example on how obtained token can be exchanged.
+
+    Arc::SecAttr* sattr = inmsg.Auth()->get("OTOKENS");    
+    if(!sattr) sattr = inmsg.AuthContext()->get("OTOKENS");
+    if(sattr) {
+      std::string token = sattr->get("");
+      if(!token.empty()) {
+        Arc::OpenIDMetadata tokenMetadata;
+        Arc::OpenIDMetadataFetcher metaFetcher(sattr->get("iss").c_str());
+        if(metaFetcher.Fetch(tokenMetadata)) {
+          char const * tokenEndpointUrl = tokenMetadata.TokenEndpoint();
+          if(tokenEndpointUrl) {
+            Arc::OpenIDTokenFetcher tokenFetcher(tokenEndpointUrl,
+                  "c85e84e8-c9ea-4ecc-8123-070df2c10e0e",
+                  "dRnakcoaT-9YA6T1LzeLAqeEu7jLBxeTWFyQMbJ6BWZonjEcE060-dn8EWAfpZmPq3x7oTjUnu6mamYylBaNhw");
+
+            std::list<std::string> scopes;
+            scopes.push_back("storage.read:/");
+            scopes.push_back("storage.create:/");
+            std::list<std::string> audiences;
+            audiences.push_back("se1.example");
+            audiences.push_back("se2.example");
+
+            Arc::OpenIDTokenFetcher::TokenList tokens;
+            if(tokenFetcher.Fetch("urn:ietf:params:oauth:grant-type:token-exchange", token, scopes, audiences, tokens)) {
+              for(auto const & token : tokens) {
+                logger_.msg(Arc::ERROR, "Token response: %s : %s", token.first, token.second);
+              };
+            } else logger_.msg(Arc::ERROR, "Failed to fetch token");
+          } else logger_.msg(Arc::ERROR, "Token metadata contains no token endpoint");;
+        } else logger_.msg(Arc::ERROR, "Failed to fetch token metadata");
+      } else logger_.msg(Arc::ERROR, "There is no token in sec attr");
+    } else logger_.msg(Arc::ERROR, "There is no otoken sec attr");
+    */
+  };
   std::string endpoint = endpoint_;
   if(endpoint.empty()) {
     std::string http_endpoint = inmsg.Attributes()->get("HTTP:ENDPOINT");
@@ -445,16 +524,27 @@ ARexConfigContext* ARexService::get_configuration(Arc::Message& inmsg) {
     };
     endpoint+=GetPath(http_endpoint);
   };
-  config=new ARexConfigContext(config_,uname,grid_name,endpoint);
+
+  // Do authorization according to allowed/denied authgroups
+  std::list<std::pair<bool,std::string> > const & groups = config_.MatchingGroups("");
+  if(!groups.empty()) {
+    if(!match_groups(groups, inmsg)) {
+      logger_.msg(Arc::ERROR, "Service access is not allowed for this user");
+      return NULL;
+    };
+  };
+
+  // Create configuration for this user
+  ARexConfigContext* config=new ARexConfigContext(config_,uname,grid_name,endpoint);
   if(config) {
     if(*config) {
       inmsg.Context()->Add("arex.gmconfig",config);
-    } else {
-      delete config; config=NULL;
-      logger_.msg(Arc::ERROR, "Failed to acquire A-REX's configuration");
+      return config;
     };
+    delete config; config=NULL;
+    logger_.msg(Arc::ERROR, "Failed to acquire A-REX's configuration");
   };
-  return config;
+  return NULL;
 }
 
 static std::string GetPath(Arc::Message &inmsg,std::string &base) {
@@ -583,18 +673,34 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
   };
 
   // Process grid-manager configuration if not done yet
-  ARexConfigContext* config = get_configuration(inmsg);
+  ARexGMConfig* config = get_configuration(inmsg);
   if(!config) {
-    logger_.msg(Arc::ERROR, "Can't obtain configuration");
-    // Service is not operational
-    char const* fault = "User can't be assigned configuration";
-    return (method == "POST") ?
-      make_soap_fault(outmsg, fault) :
-      make_http_fault(outmsg, HTTP_ERR_FORBIDDEN, fault);
+    // Service is not operational except public information.
+    // But public information also has own authorization rules
+    if(!config_.PublicInformationEnabled()) {
+      logger_.msg(Arc::VERBOSE, "Can't obtain configuration. Public information is disabled.");
+      char const* fault = "User can't be assigned configuration";
+      return (method == "POST") ?
+        make_soap_fault(outmsg, fault) :
+        make_http_fault(outmsg, HTTP_ERR_FORBIDDEN, fault);
+    };
+    // Check additional authorization rules
+    std::list<std::pair<bool,std::string> > const & groups = config_.MatchingGroupsPublicInformation();
+    if(!groups.empty()) {
+      if(!match_groups(groups, inmsg)) {
+        logger_.msg(Arc::VERBOSE, "Can't obtain configuration. Public information is disallowed for this user.");
+        char const* fault = "User can't be assigned configuration";
+        return (method == "POST") ?
+          make_soap_fault(outmsg, fault) :
+          make_http_fault(outmsg, HTTP_ERR_FORBIDDEN, fault);
+      };
+    };
+    logger_.msg(Arc::VERBOSE, "Can't obtain configuration. Only public information is provided.");
+  } else {
+    config->ClearAuths();
+    config->AddAuth(inmsg.Auth());
+    config->AddAuth(inmsg.AuthContext());
   };
-  config->ClearAuths();
-  config->AddAuth(inmsg.Auth());
-  config->AddAuth(inmsg.AuthContext());
 
   // Identify which of served endpoints request is for.
   // Using simplified algorithm - POST for SOAP messages,
@@ -607,7 +713,16 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
       logger_.msg(Arc::ERROR, "POST request on special path is not supported");
       return make_soap_fault(outmsg);
     };
-    if(id.empty()) {
+    if (!id.empty()) {
+      // Listing operations for session directories
+      // TODO: proper failure like interface is not supported
+      logger_.msg(Arc::ERROR, "Per-job POST/SOAP requests are not supported");
+      return make_soap_fault(outmsg,"Operation not supported");
+    };
+    {
+      // TODO: Split this complex code into separate methods. Make handling of anonymous (config == null) 
+      // requests processing cleaner.
+
       // Factory operations
       logger_.msg(Arc::VERBOSE, "process: factory endpoint");
       Arc::PayloadSOAP* outpayload = new Arc::PayloadSOAP(ns_);
@@ -616,6 +731,7 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
       outpayload->Namespaces(ns_);
       if(config_.EMIESInterfaceEnabled() && MatchXMLNamespace(op,ES_CREATE_NAMESPACE)) {
         // Aplying known namespaces
+        if(!config) return make_soap_fault(outmsg, "User can't be assigned configuration");
         inpayload->Namespaces(ns_);
         if(MatchXMLName(op,"CreateActivity")) {
           CountedResourceLock cl_lock(beslimit_);
@@ -625,6 +741,7 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
         }
       } else if(config_.EMIESInterfaceEnabled() && MatchXMLNamespace(op,ES_RINFO_NAMESPACE)) {
         // Aplying known namespaces
+        // Resource information is available for anonymous requests - null config is handled properly
         inpayload->Namespaces(ns_);
         if(MatchXMLName(op,"GetResourceInfo")) {
           CountedResourceLock cl_lock(infolimit_);
@@ -636,6 +753,7 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
           SOAP_NOT_SUPPORTED;
         }
       } else if(config_.EMIESInterfaceEnabled() && MatchXMLNamespace(op,ES_MANAG_NAMESPACE)) {
+        if(!config) return make_soap_fault(outmsg, "User can't be assigned configuration");
         // Aplying known namespaces
         inpayload->Namespaces(ns_);
         if(MatchXMLName(op,"PauseActivity")) {
@@ -666,6 +784,7 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
           SOAP_NOT_SUPPORTED;
         }
       } else if(config_.EMIESInterfaceEnabled() && MatchXMLNamespace(op,ES_AINFO_NAMESPACE)) {
+        if(!config) return make_soap_fault(outmsg, "User can't be assigned configuration");
         // Aplying known namespaces
         inpayload->Namespaces(ns_);
         if(MatchXMLName(op,"ListActivities")) {
@@ -681,6 +800,7 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
           SOAP_NOT_SUPPORTED;
         }
       } else if(config_.ARCInterfaceEnabled() && MatchXMLNamespace(op,BES_ARC_NAMESPACE)) {
+        if(!config) return make_soap_fault(outmsg, "User can't be assigned configuration");
         // Aplying known namespaces
         inpayload->Namespaces(ns_);
         if(MatchXMLName(op,"CacheCheck")) {
@@ -689,6 +809,7 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
           SOAP_NOT_SUPPORTED;
         }
       } else if(delegation_stores_.MatchNamespace(*inpayload)) {
+        if(!config) return make_soap_fault(outmsg, "User can't be assigned configuration");
         // Aplying known namespaces
         inpayload->Namespaces(ns_);
         CountedResourceLock cl_lock(beslimit_);
@@ -749,11 +870,6 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
         logger_.msg(Arc::VERBOSE, "process: response=%s",str);
       };
       outmsg.Payload(outpayload);
-    } else {
-      // Listing operations for session directories
-      // TODO: proper failure like interface is not supported
-      logger_.msg(Arc::ERROR, "Per-job POST/SOAP requests are not supported");
-      return make_soap_fault(outmsg,"Operation not supported");
     };
     Arc::MCC_Status sret = ProcessSecHandlers(outmsg,"outgoing");
     if(!sret) {
@@ -799,7 +915,10 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
     };
     return ret;
   } else if(method == "HEAD") {
+    logger_.msg(Arc::VERBOSE, "process: HEAD");
+    logger_.msg(Arc::INFO, "HEAD: id %s path %s", id, subpath);
     Arc::MCC_Status ret;
+    CountedResourceLock cl_lock(datalimit_);
     switch(sub_op) {
       case SubOpInfo:
         ret = HeadInfo(inmsg,outmsg,*config,subpath);
@@ -913,6 +1032,9 @@ Arc::MCC_Status ARexService::HeadDelegation(Arc::Message& inmsg,Arc::Message& ou
 }
 
 Arc::MCC_Status ARexService::GetDelegation(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string const& id,std::string const& subpath) {
+  if(!&config) {
+    return make_http_fault(outmsg, HTTP_ERR_FORBIDDEN, "User is not identified");
+  };
   if(!subpath.empty()) {
     return make_http_fault(outmsg,500,"No additional path expected");
   };
