@@ -16,7 +16,6 @@
 namespace ARex {
 
 class ARexGMConfig;
-class ARexConfigContext;
 class CountedResourceLock;
 
 class CountedResource {
@@ -78,7 +77,7 @@ class ARexService: public Arc::Service {
   FileChunksList files_chunks_;
   GMConfig config_;
   GridManager* gm_;
-  ARexConfigContext* get_configuration(Arc::Message& inmsg);
+  ARexGMConfig* get_configuration(Arc::Message& inmsg);
 
   // A-REX operations
   AREXOP(CacheCheck);
@@ -104,6 +103,7 @@ class ARexService: public Arc::Service {
   Arc::MCC_Status GetJob(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string const& id,std::string const& subpath);
   Arc::MCC_Status GetLogs(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string const& id,std::string const& subpath);
   Arc::MCC_Status GetInfo(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string const& subpath);
+  Arc::MCC_Status GetInfo(Arc::Message& inmsg,Arc::Message& outmsg);
   Arc::MCC_Status GetNew(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string const& subpath);
   Arc::MCC_Status GetDelegation(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string const& id,std::string const& subpath);
   Arc::MCC_Status GetCache(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string const& subpath);
@@ -111,6 +111,7 @@ class ARexService: public Arc::Service {
   Arc::MCC_Status HeadJob(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string const& id,std::string const& subpath);
   Arc::MCC_Status HeadLogs(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string const& id,std::string const& subpath);
   Arc::MCC_Status HeadInfo(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string const& subpath);
+  Arc::MCC_Status HeadInfo(Arc::Message& inmsg,Arc::Message& outmsg);
   Arc::MCC_Status HeadNew(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string const& subpath);
   Arc::MCC_Status HeadDelegation(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string const& id,std::string const& subpath);
   Arc::MCC_Status HeadCache(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string const& subpath);
