@@ -188,7 +188,8 @@ bool job_log_make_file(const GMJob &job,const GMConfig& config,const std::string
       std::string value(line->substr(p+1));
       if (key.empty()) continue;
       if (key == "nodename") {
-        if (nodecount) nodenames+=":"; nodenames+=value;
+        if (nodecount) nodenames+=":";
+        nodenames+=value;
         nodecount++;
       } else if(key == "processors") {
         job_data += "processors=" + value + '\n';
