@@ -111,28 +111,6 @@ class DirectUserFilePlugin: public DirectFilePlugin {
 
 };
 
-
-static void job_subst(std::string& str,void* arg) {
-  job_subst_t* subs = (job_subst_t*)arg;
-  if(subs->job) for(std::string::size_type p = 0;;) {
-    p=str.find('%',p);
-    if(p==std::string::npos) break;
-    if(str[p+1]=='I') {
-      str.replace(p,2,subs->job->c_str());
-      p+=subs->job->length();
-    } else if(str[p+1]=='S') {
-      str.replace(p,2,"UNKNOWN"); // WRONG
-      p+=7;
-    } else if(str[p+1]=='O') {
-      str.replace(p,2,subs->reason);
-      p+=strlen(subs->reason);
-    } else {
-      p+=2;
-    };
-  };
-  if(subs->user && subs->config) subs->config->Substitute(str, *(subs->user));
-}
-
 JobPlugin::JobPlugin(std::istream &cfile,userspec_t const &user_,FileNode& node):
     cont_plugins(new ContinuationPlugins),
     user_s(user_),
@@ -986,7 +964,7 @@ int JobPlugin::close(bool eof) {
   /* **********************************************************
    * Create status file (do it last so GM picks job up here)  *
    ********************************************************** */
-  if(!job_state_write_file(job,config,JOB_STATE_ACCEPTED)) {
+  if(!job_state_write_file(job,config,JOB_STATE_ACCEPTED,false)) {
     logger.msg(Arc::ERROR, "Failed writing status");
     delete_job_id(); 
     error_description="Failed registering job in A-REX.";
@@ -1377,7 +1355,6 @@ bool JobPlugin::make_job_id(const std::string &id) {
 }
 
 bool JobPlugin::make_job_id(void) {
-  bool found = false;
   delete_job_id();
   for(int i=0;i<100;i++) {
     //std::string id=Arc::tostring((unsigned int)getpid())+
