@@ -44,7 +44,7 @@ bool GMJobQueueDTR::CanRemove(GMJob const& job) {
 
 Arc::Logger DTRGenerator::logger(Arc::Logger::getRootLogger(), "Generator");
 
-bool compare_job_description(GMJobRef const& first, GMJobRef const& second) {
+bool compare_job_description(GMJob const * first, GMJob const * second) {
   if(!first) return false;
   if(!second) return false;
   int priority_first = first->GetLocalDescription() ? first->GetLocalDescription()->priority : JobLocalDescription::prioritydefault;
@@ -70,30 +70,47 @@ void DTRGenerator::thread() {
     // look at event queue and deal with any events.
     // This method of iteration should be thread-safe because events
     // are always added to the end of the list
+    logger.msg(Arc::DEBUG, "DTR Generator waiting to process: %d jobs to cancel, %d DTRs, %d new jobs",
+                                 jobs_cancelled.size(), dtrs_received.size(), jobs_received.Size());
+    int cancelled_num = 0;
+    int dtrs_num = 0;
+    int jobs_num = 0;
 
     // take cancelled jobs first so we can ignore other DTRs in those jobs
-    event_lock.lock();
+    Arc::AutoLock<Arc::SimpleCondition> elock(event_lock);
     std::list<std::string>::iterator it_cancel = jobs_cancelled.begin();
     while (it_cancel != jobs_cancelled.end()) {
       // check if it is still in received queue and remove
-      if(!jobs_received.Erase(*it_cancel)) {
+      GMJobRef job = jobs_received.Find(*it_cancel);
+      if(!job) {
         // job must be in scheduler already
-        event_lock.unlock();
+        logger.msg(Arc::DEBUG, "%s: Job cancel request from DTR generator to scheduler", *it_cancel);
+        elock.unlock();
         processCancelledJob(*it_cancel);
-        event_lock.lock();
+        elock.lock();
+      } else {
+        logger.msg(Arc::DEBUG, "%s: Returning canceled job from DTR generator", job->get_id());
+        elock.unlock();
+        {
+          Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
+          finished_jobs[job->get_id()] = std::string("Job was canceled while waiting in DTR queue");
+        }
+        elock.lock();
+        jobs_received.Erase(job);
+        jobs.RequestAttention(job); // pass job back to states processing
       }
       it_cancel = jobs_cancelled.erase(it_cancel);
+      ++cancelled_num;
     }
 
     // next DTRs sent back from the Scheduler
     std::list<DataStaging::DTR_ptr>::iterator it_dtrs = dtrs_received.begin();
     while (it_dtrs != dtrs_received.end()) {
-      event_lock.unlock();
+      elock.unlock();
       processReceivedDTR(*it_dtrs);
-      event_lock.lock();
-      // delete DTR LogDestinations
-      (*it_dtrs)->clean_log_destinations(central_dtr_log);
+      elock.lock();
       it_dtrs = dtrs_received.erase(it_dtrs);
+      ++dtrs_num;
     }
 
     // finally new jobs
@@ -108,18 +125,27 @@ void DTRGenerator::thread() {
     while (Arc::Time() < limit) {
       GMJobRef job = jobs_received.Front(); // get reference but keep job in queue
       if(!job) break;
-      event_lock.unlock();
+      elock.unlock();
       bool jobAccepted = processReceivedJob(job);
-      event_lock.lock();
+      // on success job is moved to jobs_processing queue
+      elock.lock();
       if(!jobAccepted) {
         logger.msg(Arc::DEBUG, "%s: Re-requesting attention from DTR generator", job->get_id());
+        // processReceivedJob fills error in finished_jobs - no need to do that here
         jobs_received.Erase(job); // release from queue cause 'jobs' queues have lower priority
         jobs.RequestAttention(job); // pass job back to states processing
       }
+      ++jobs_num;
     }
-    event_lock.unlock();
-    event_lock.wait(50000);
-  }
+    bool queuesEmpty = jobs_cancelled.empty() && dtrs_received.empty() && jobs_received.IsEmpty();
+    elock.unlock();
+
+    logger.msg(Arc::DEBUG, "DTR Generator processed: %d jobs to cancel, %d DTRs, %d new jobs",
+                                 cancelled_num, dtrs_num, jobs_num);
+
+    // wait till something arrives or go back to processing almost immediately if queues not empty
+    event_lock.wait(queuesEmpty ? 50000 : 100);
+  } // main processing loop
   // stop scheduler - cancels all DTRs and waits for them to complete
   scheduler->stop();
   // Handle all the DTRs returned by the scheduler, in case there are completed
@@ -130,8 +156,6 @@ void DTRGenerator::thread() {
   std::list<DataStaging::DTR_ptr>::iterator it_dtrs = dtrs_received.begin();
   while (it_dtrs != dtrs_received.end()) {
     processReceivedDTR(*it_dtrs);
-    // delete DTR LogDestinations
-    (*it_dtrs)->clean_log_destinations(central_dtr_log);
     it_dtrs = dtrs_received.erase(it_dtrs);
   }
   run_condition.signal();
@@ -229,57 +253,68 @@ void DTRGenerator::receiveDTR(DataStaging::DTR_ptr dtr) {
     logger.msg(Arc::VERBOSE, "Received DTR %s during Generator shutdown - may not be processed", dtr->get_id());
     // still a chance to process this DTR so don't return
   }
-  event_lock.lock();
+  Arc::AutoLock<Arc::SimpleCondition> elock(event_lock);
   dtrs_received.push_back(dtr);
   event_lock.signal_nonblock();
-  event_lock.unlock();
 }
 
-void DTRGenerator::receiveJob(GMJobRef& job) {
-
+bool DTRGenerator::receiveJob(GMJobRef& job) {
   if (generator_state != DataStaging::RUNNING) {
     logger.msg(Arc::WARNING, "DTRGenerator is not running!");
   }
 
+  if(!job) {
+    logger.msg(Arc::ERROR, "DTRGenerator was sent null job");
+    return false;
+  }
+
   // Add to jobs list even if Generator is stopped, so that A-REX doesn't
   // think that staging has finished.
-  event_lock.lock();
-  jobs_received.PushSorted(job, compare_job_description);
-  event_lock.signal_nonblock();
-  event_lock.unlock();
+  Arc::AutoLock<Arc::SimpleCondition> elock(event_lock);
+  bool result = jobs_received.PushSorted(job, compare_job_description);
+  if(result) {
+    logger.msg(Arc::DEBUG, "%s: Received job in DTR generator", job->get_id());
+    event_lock.signal_nonblock();
+  } else {
+    logger.msg(Arc::ERROR, "%s: Failed to receive job in DTR generator", job->get_id());
+  }
+  return result;
 }
 
 void DTRGenerator::cancelJob(const GMJobRef& job) {
-  if(!job) return;
+  if(!job) {
+    logger.msg(Arc::ERROR, "DTRGenerator got request to cancel null job");
+    return;
+  }
 
   if (generator_state != DataStaging::RUNNING) {
     logger.msg(Arc::WARNING, "DTRGenerator is not running!");
   }
 
-  event_lock.lock();
+  Arc::AutoLock<Arc::SimpleCondition> elock(event_lock);
   jobs_cancelled.push_back(job->get_id());
   event_lock.signal_nonblock();
-  event_lock.unlock();
 }
 
 bool DTRGenerator::queryJobFinished(GMJobRef const& job) {
-  if(!job) return false;
+  if(!job) {
+    logger.msg(Arc::ERROR, "DTRGenerator is queried about null job");
+    return false;
+  }
 
   // Data staging is finished if the job is in finished_jobs and
   // not in active_dtrs or jobs_received.
 
   // check if this job is still in the received jobs queue
-  event_lock.lock();
+  Arc::AutoLock<Arc::SimpleCondition> elock(event_lock);
   if(jobs_received.Exists(job)) {
-    event_lock.unlock();
     return false;
   }
-  event_lock.unlock();
+  elock.unlock();
 
   // check if any DTRs in this job are still active
-  dtrs_lock.lock();
+  Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
   if (active_dtrs.find(job->get_id()) != active_dtrs.end()) {
-    dtrs_lock.unlock();
     return false;
   }
   std::map<std::string, std::string>::iterator i = finished_jobs.find(job->get_id());
@@ -288,55 +323,54 @@ bool DTRGenerator::queryJobFinished(GMJobRef const& job) {
     job->AddFailure(i->second);
     finished_jobs[job->get_id()] = "";
   }
-  dtrs_lock.unlock();
   return true;
 }
 
 bool DTRGenerator::hasJob(const GMJobRef& job) {
-  if(!job) return false;
+  if(!job) {
+    logger.msg(Arc::ERROR, "DTRGenerator is asked about null job");
+    return false;
+  }
 
   // check if this job is still in the received jobs queue
-  event_lock.lock();
+  Arc::AutoLock<Arc::SimpleCondition> elock(event_lock);
   if(jobs_received.Exists(job)) {
-    event_lock.unlock();
     return true;
   }
-  event_lock.unlock();
+  elock.unlock();
 
   // check if any DTRs in this job are still active
-  dtrs_lock.lock();
+  Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
   if (active_dtrs.find(job->get_id()) != active_dtrs.end()) {
-    dtrs_lock.unlock();
     return true;
   }
 
   // finally check finished jobs
   std::map<std::string, std::string>::iterator i = finished_jobs.find(job->get_id());
   if (i != finished_jobs.end()) {
-    dtrs_lock.unlock();
     return true;
   }
-  dtrs_lock.unlock();
   // not found
   return false;
 }
 
 void DTRGenerator::removeJob(const GMJobRef& job) {
-  if(!job) return;
+  if(!job) {
+    logger.msg(Arc::ERROR, "DTRGenerator is requested to remove null job");
+    return;
+  }
 
   // check if this job is still in the received jobs queue
-  event_lock.lock();
+  Arc::AutoLock<Arc::SimpleCondition> elock(event_lock);
   if(jobs_received.Exists(job)) {
-    event_lock.unlock();
     logger.msg(Arc::WARNING, "%s: Trying to remove job from data staging which is still active", job->get_id());
     return;
   }
-  event_lock.unlock();
+  elock.unlock();
 
   // check if any DTRs in this job are still active
-  dtrs_lock.lock();
+  Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
   if (active_dtrs.find(job->get_id()) != active_dtrs.end()) {
-    dtrs_lock.unlock();
     logger.msg(Arc::WARNING, "%s: Trying to remove job from data staging which is still active", job->get_id());
     return;
   }
@@ -345,39 +379,43 @@ void DTRGenerator::removeJob(const GMJobRef& job) {
   std::map<std::string, std::string>::iterator i = finished_jobs.find(job->get_id());
   if (i == finished_jobs.end()) {
     // warn if not in finished
-    dtrs_lock.unlock();
     logger.msg(Arc::WARNING, "%s: Trying remove job from data staging which does not exist", job->get_id());
     return;
   }
   finished_jobs.erase(i);
-  dtrs_lock.unlock();
 }
 
 bool DTRGenerator::processReceivedDTR(DataStaging::DTR_ptr dtr) {
 
   std::string jobid(dtr->get_parent_job_id());
+  GMJobRef job = jobs_processing.Find(jobid);
   if (!(*dtr)) {
     logger.msg(Arc::ERROR, "%s: Invalid DTR", jobid);
     if (dtr->get_status() != DataStaging::DTRStatus::CANCELLED) {
       scheduler->cancelDTRs(jobid);
-      dtrs_lock.lock();
-      finished_jobs[jobid] = std::string("Invalid Data Transfer Request");
-      active_dtrs.erase(jobid);
-      dtrs_lock.unlock();
+      {
+        Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
+        finished_jobs[jobid] = std::string("Invalid Data Transfer Request");
+        active_dtrs.erase(jobid);
+      }
+      // Because it is not possible to find out if there will be more 
+      // job's DTR coming, if possible return job back to jobs processing queue.
+      if(job) {
+        jobs_processing.Erase(job);
+        jobs.RequestAttention(job);
+      }
     }
     return false;
   }
   logger.msg(Arc::DEBUG, "%s: Received DTR %s to copy file %s in state %s",
                           jobid, dtr->get_id(), dtr->get_source()->str(), dtr->get_status().str());
-  GMJobRef job = jobs_processing.Find(jobid);
   if(!job) {
     // This job is not being processed anymore (somehow)
     logger.msg(Arc::ERROR, "%s: Received DTR belongs to inactive job", jobid);
     scheduler->cancelDTRs(jobid); // Cancel rest of such DTRs
-    dtrs_lock.lock();
+    Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
     finished_jobs[jobid] = std::string("Job was gone while performing data transfer");
     active_dtrs.erase(jobid);
-    dtrs_lock.unlock();
     return false;
   }
 
@@ -402,7 +440,7 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR_ptr dtr) {
     logger.msg(Arc::ERROR, "%s: DTR %s to copy file %s failed",
                jobid, dtr->get_id(), dtr->get_source()->str());
 
-    dtrs_lock.lock();
+    Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
     if (!dtr->get_source()->Local() && finished_jobs.find(jobid) == finished_jobs.end()) { // download
       // cancel other DTRs and erase from our list unless error was already reported
       logger.msg(Arc::INFO, "%s: Cancelling other DTRs", jobid);
@@ -410,7 +448,6 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR_ptr dtr) {
     }
     // add error to finished jobs
     finished_jobs[jobid] += std::string("Failed in data staging: " + dtr->get_error_status().GetDesc() + '\n');
-    dtrs_lock.unlock();
   }
   else if (dtr->get_status() != DataStaging::DTRStatus::CANCELLED) {
     // remove from job.id.input/output files on success
@@ -462,7 +499,7 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR_ptr dtr) {
           } else {
             ++i;
           }
-        }
+        } // files
 
         // write back .output file
         if (!job_output_write_file(*job, config, files)) {
@@ -518,13 +555,17 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR_ptr dtr) {
   }
 
   // get DTRs for this job id
-  dtrs_lock.lock();
+  Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
   std::pair<std::multimap<std::string, std::string>::iterator,
             std::multimap<std::string, std::string>::iterator> dtr_iterator = active_dtrs.equal_range(jobid);
 
   if (dtr_iterator.first == dtr_iterator.second) {
-    dtrs_lock.unlock();
+    finished_jobs[jobid] += std::string(""); // It is not clear either this is error. At least mark it as finished.
+    dlock.unlock();
     logger.msg(Arc::WARNING, "No active job id %s", jobid);
+    // No DTRs recorded. But still we have job ref. It is probably safer to return it.
+    jobs_processing.Erase(job);
+    jobs.RequestAttention(job);
     return true;
   }
   
@@ -546,7 +587,7 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR_ptr dtr) {
 
   // check if any DTRs left from this job, if so return
   if (active_dtrs.find(jobid) != active_dtrs.end()) {
-    dtrs_lock.unlock();
+    // still have some DTRs running
     return true;
   }
 
@@ -559,7 +600,7 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR_ptr dtr) {
   bool finished_with_error = ((finished_jobs.find(jobid) != finished_jobs.end() &&
                                !finished_jobs[jobid].empty()) ||
                               dtr->get_status() == DataStaging::DTRStatus::CANCELLED);
-  dtrs_lock.unlock();
+  dlock.unlock();
 
   if (dtr->get_source()->Local()) {
     // list of files to keep in session dir
@@ -616,19 +657,19 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR_ptr dtr) {
   }
   // add to finished jobs (without overwriting existing error) and finally
   // remove from active
-  dtrs_lock.lock();
+  dlock.lock();
   active_dtrs.erase(jobid);
   finished_jobs[jobid] += "";
 
   // log summary to DTR log and A-REX log
   if (finished_jobs[jobid].empty())
     dtr->get_logger()->msg(Arc::INFO, "%s: All %s %s successfully", jobid,
-                          dtr->get_source()->Local() ? "uploads":"downloads",
-                          (dtr->get_status() == DataStaging::DTRStatus::CANCELLED) ? "cancelled":"finished");
+                           dtr->get_source()->Local() ? istring("uploads") : istring("downloads"),
+                           (dtr->get_status() == DataStaging::DTRStatus::CANCELLED) ? istring("cancelled") : istring("finished"));
   else
     dtr->get_logger()->msg(Arc::INFO, "%s: Some %s failed", jobid,
-                          dtr->get_source()->Local() ? "uploads":"downloads");
-  dtrs_lock.unlock();
+                           dtr->get_source()->Local() ? istring("uploads") : istring("downloads"));
+  dlock.unlock();
 
   logger.msg(Arc::DEBUG, "%s: Requesting attention from DTR generator", jobid);
   // Passing job to lower priority queue - hence must use Erase.
@@ -640,11 +681,14 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR_ptr dtr) {
 
 
 bool DTRGenerator::processReceivedJob(GMJobRef& job) {
-  if(!job) return false;
+  if(!job) {
+    logger.msg(Arc::ERROR, "DTRGenerator is requested to process null job");
+    return false;
+  }
 
   JobId jobid(job->get_id());
   logger.msg(Arc::VERBOSE, "%s: Received data staging request to %s files", jobid,
-             (job->get_state() == JOB_STATE_PREPARING ? "download" : "upload"));
+             (job->get_state() == JOB_STATE_PREPARING ? istring("download") : istring("upload")));
 
   uid_t job_uid = config.StrictSession() ? job->get_user().get_uid() : 0;
   uid_t job_gid = config.StrictSession() ? job->get_user().get_gid() : 0;
@@ -682,19 +726,20 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
   std::list<FileData> output_files;
   if (!job_output_read_file(jobid, config, output_files)) {
     logger.msg(Arc::ERROR, "%s: Failed to read list of output files", jobid);
-    dtrs_lock.lock();
-    finished_jobs[jobid] = std::string("Failed to read list of output files");
-    dtrs_lock.unlock();
+    {
+      Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
+      finished_jobs[jobid] = std::string("Failed to read list of output files");
+    }
     if (job->get_state() == JOB_STATE_FINISHING) CleanCacheJobLinks(config, job);
     return false;
   }
 
+  Arc::Time sessiondir_processing_start;
   if (job->get_state() == JOB_STATE_PREPARING) {
     if (!job_input_read_file(jobid, config, files)) {
       logger.msg(Arc::ERROR, "%s: Failed to read list of input files", jobid);
-      dtrs_lock.lock();
+      Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
       finished_jobs[jobid] = std::string("Failed to read list of input files");
-      dtrs_lock.unlock();
       return false;
     }
     // check for duplicates (see bug 1285)
@@ -702,9 +747,8 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
       for (std::list<FileData>::iterator j = files.begin(); j != files.end(); j++) {
         if (i != j && j->pfn == i->pfn) {
           logger.msg(Arc::ERROR, "%s: Duplicate file in list of input files: %s", jobid, i->pfn);
-          dtrs_lock.lock();
+          Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
           finished_jobs[jobid] = std::string("Duplicate file in list of input files: " + i->pfn);
-          dtrs_lock.unlock();
           return false;
         }
       }
@@ -732,12 +776,11 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
     }
     if (!Arc::DirDeleteExcl(job->SessionDir(), todelete, false, job_uid, job_gid)) {
       logger.msg(Arc::ERROR, "%s: Failed to clean up session dir", jobid);
-      dtrs_lock.lock();
+      Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
       finished_jobs[jobid] = std::string("Failed to clean up session dir before downloading inputs");
-      dtrs_lock.unlock();
       return false;
     }
-  }
+  } // PREPARING
   else if (job->get_state() == JOB_STATE_FINISHING) {
     files = output_files;
     std::list<FileData>::iterator it;
@@ -755,14 +798,14 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
         logger.msg(Arc::INFO, "%s: Reading output files from user generated list in %s", jobid, outputfilelist);
         if (!job_Xput_read_file(outputfilelist, files_, job_uid, job_gid)) {
           logger.msg(Arc::ERROR, "%s: Error reading user generated output file list in %s", jobid, outputfilelist);
-          dtrs_lock.lock();
+          Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
           // Only write this failure if no previous failure
           if (!job->CheckFailure(config)) {
             finished_jobs[jobid] = std::string("Error reading user generated output file list");
           } else {
             finished_jobs[jobid] = "";
           }
-          dtrs_lock.unlock();
+          dlock.unlock();
           CleanCacheJobLinks(config, job);
           return false;
         }
@@ -782,14 +825,14 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
           std::list<std::string> entries;
           if (!Arc::DirList(dir, entries, true, job_uid, job_gid)) {
             logger.msg(Arc::ERROR, "%s: Failed to list output directory %s: %s", jobid, dir, Arc::StrError(errno));
-            dtrs_lock.lock();
+            Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
             // Only write this failure if no previous failure
             if (!job->CheckFailure(config)) {
               finished_jobs[jobid] = std::string("Failed to list output directory");
             } else {
               finished_jobs[jobid] = "";
             }
-            dtrs_lock.unlock();
+            dlock.unlock();
             CleanCacheJobLinks(config, job);
             return false;
           }
@@ -822,9 +865,10 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
           // error if lfns (including locations) are identical
           if (it->lfn == it2->lfn) {
             logger.msg(Arc::ERROR, "%s: Two identical output destinations: %s", jobid, it->lfn);
-            dtrs_lock.lock();
-            finished_jobs[jobid] = std::string("Two identical output destinations: " + it->lfn);
-            dtrs_lock.unlock();
+            {
+              Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
+              finished_jobs[jobid] = std::string("Two identical output destinations: " + it->lfn);
+            }
             CleanCacheJobLinks(config, job);
             return false;
           }
@@ -834,9 +878,10 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
             // error if pfns are different
             if (it->pfn != it2->pfn) {
               logger.msg(Arc::ERROR, "%s: Cannot upload two different files %s and %s to same LFN: %s", jobid, it->pfn, it2->pfn, it->lfn);
-              dtrs_lock.lock();
-              finished_jobs[jobid] = std::string("Cannot upload two different files to same LFN: " + it->lfn);
-              dtrs_lock.unlock();
+              {
+                Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
+                finished_jobs[jobid] = std::string("Cannot upload two different files to same LFN: " + it->lfn);
+              }
               CleanCacheJobLinks(config, job);
               return false;
             }
@@ -854,21 +899,27 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
     std::transform(files.begin(), files.end(), tokeep.begin(), filedata_pfn);
     if (!Arc::DirDeleteExcl(job->SessionDir(), tokeep, true, job_uid, job_gid)) {
       logger.msg(Arc::ERROR, "%s: Failed to clean up session dir", jobid);
-      dtrs_lock.lock();
-      finished_jobs[jobid] = std::string("Failed to clean up session dir before uploading outputs");
-      dtrs_lock.unlock();
+      {
+        Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
+        finished_jobs[jobid] = std::string("Failed to clean up session dir before uploading outputs");
+      }
       CleanCacheJobLinks(config, job);
       return false;
     }
-  }
+  } // FINISHING
   else {
     // bad state
     logger.msg(Arc::ERROR, "%s: Received job in a bad state: %s", jobid, job->get_state_name());
-    dtrs_lock.lock();
+    Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
     finished_jobs[jobid] = std::string("Logic error: DTR Generator received job in a bad state");
-    dtrs_lock.unlock();
     return false;
   }
+  Arc::Time sessiondir_processing_end;
+  Arc::Period sessiondir_processing_time = sessiondir_processing_end - sessiondir_processing_start;
+  if ((sessiondir_processing_time.GetPeriod() >= 1) || (sessiondir_processing_time.GetPeriodNanoseconds() > 100000000)) { // >0.1s
+    logger.msg(Arc::WARNING, "%s: Session directory processing takes too long - %u.%06u seconds",
+                               jobid, sessiondir_processing_time.GetPeriod(), sessiondir_processing_time.GetPeriodNanoseconds()/1000);
+  }
   Arc::initializeCredentialsType cred_type(Arc::initializeCredentialsType::SkipCredentials);
   Arc::UserConfig usercfg(cred_type);
   usercfg.UtilsDirPath(config.ControlDir());
@@ -883,9 +934,8 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
     // if job is FINISHING then clean up cache joblinks
     if (job->get_state() == JOB_STATE_FINISHING) CleanCacheJobLinks(config, job);
     // nothing else to do so wake up GM thread and return
-    dtrs_lock.lock();
+    Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
     finished_jobs[jobid] = "";
-    dtrs_lock.unlock();
     return false;
   }
 
@@ -920,8 +970,8 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
         source = i->lfn;
       }
       destination = "file:" + job->SessionDir() + i->pfn;
-    }
-    else {
+    } // PREPARING
+    else { // FINISHING
       source = "file:" + job->SessionDir() + i->pfn;
       // Upload to dest ending in '/': append filename to lfn
       // Note: won't work for nested URLs used for index services
@@ -963,18 +1013,16 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
       usercfg.ProxyPath(default_cred);
       if (Arc::FileRead(default_cred, proxy_cred)) usercfg.CredentialString(proxy_cred);
     }
-    // logger for these DTRs. LogDestinations should be deleted when DTR is received back
-    DataStaging::DTRLogger dtr_log(new Arc::Logger(Arc::Logger::getRootLogger(), "DataStaging.DTR"));
-    Arc::LogFile * dest = new Arc::LogFile(job_errors_filename(jobid, config));
+
+    std::list<DataStaging::DTRLogDestination> logs;
+    Arc::LogFile* dest = new Arc::LogFile(job_errors_filename(jobid, config));
     dest->setReopen(true);
     dest->setFormat(Arc::MediumFormat);
-    dtr_log->addDestination(*dest);
-    if (central_dtr_log) {
-      dtr_log->addDestination(*central_dtr_log);
-    }
+    logs.push_back(dest);
+    if (central_dtr_log) logs.push_back(central_dtr_log);
 
     // create DTR and send to Scheduler
-    DataStaging::DTR_ptr dtr(new DataStaging::DTR(source, destination, usercfg, jobid, job->get_user().get_uid(), dtr_log));
+    DataStaging::DTR_ptr dtr(new DataStaging::DTR(source, destination, usercfg, jobid, job->get_user().get_uid(), logs, "DataStaging.DTR"));
     // set retry count (tmp errors only)
     dtr->set_tries_left(staging_conf.max_retries);
     // allow the same file to be uploaded to multiple locations with same LFN
@@ -999,15 +1047,17 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
     // Substitute cache paths
     cache_params.substitute(config, job->get_user());
     cache_parameters.cache_dirs = cache_params.getCacheDirs();
+    cache_parameters.readonly_cache_dirs = cache_params.getReadOnlyCacheDirs();
     dtr->set_cache_parameters(cache_parameters);
     dtr->registerCallback(this,DataStaging::GENERATOR);
     dtr->registerCallback(scheduler, DataStaging::SCHEDULER);
     // callbacks for info
     dtr->registerCallback(&info, DataStaging::SCHEDULER);
     dtr->set_credential_info(cred_info);
-    dtrs_lock.lock();
-    active_dtrs.insert(std::pair<std::string, std::string>(jobid, dtr->get_id()));
-    dtrs_lock.unlock();
+    {
+      Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
+      active_dtrs.insert(std::pair<std::string, std::string>(jobid, dtr->get_id()));
+    }
     // send to Scheduler
     DataStaging::DTR::push(dtr, DataStaging::SCHEDULER);
 
@@ -1023,13 +1073,12 @@ bool DTRGenerator::processReceivedJob(GMJobRef& job) {
       logger.msg(Arc::ERROR, "%s: Failed writing local information", jobid);
     }
     delete job_desc;
-  }
+  } // files
   if (!staging) { // nothing needed staged so mark as finished
     // if job is FINISHING then clean up cache joblinks
     if (job->get_state() == JOB_STATE_FINISHING) CleanCacheJobLinks(config, job);
-    dtrs_lock.lock();
+    Arc::AutoLock<Arc::SimpleCondition> dlock(dtrs_lock);
     finished_jobs[jobid] = "";
-    dtrs_lock.unlock();
     return false;
   }
   jobs_processing.Push(job); // take this job (job should be in jobs_received till now)
@@ -1045,7 +1094,10 @@ bool DTRGenerator::processCancelledJob(const std::string& jobid) {
 }
 
 DTRGenerator::checkUploadedFilesResult DTRGenerator::checkUploadedFiles(GMJobRef& job) {
-  if(!job) return uploadedFilesError;
+  if(!job) {
+    logger.msg(Arc::ERROR, "DTRGenerator is asked to check files for null job");
+    return uploadedFilesError;
+  }
 
   JobId jobid(job->get_id());
   uid_t job_uid = config.StrictSession() ? job->get_user().get_uid() : 0;
@@ -1296,15 +1348,27 @@ void DTRGenerator::readDTRState(const std::string& dtr_log) {
 }
 
 void DTRGenerator::CleanCacheJobLinks(const GMConfig& config, const GMJobRef& job) const {
-  if(!job) return;
+  if(!job) {
+    logger.msg(Arc::ERROR, "DTRGenerator is requested to clean links for null job");
+    return;
+  }
 
+  Arc::Time processing_start;
   CacheConfig cache_config(config.CacheParams());
   cache_config.substitute(config, job->get_user());
   // there is no uid switch during Release so uid/gid is not so important
   Arc::FileCache cache(cache_config.getCacheDirs(),
                        cache_config.getDrainingCacheDirs(),
+                       cache_config.getReadOnlyCacheDirs(),
                        job->get_id(), job->get_user().get_uid(), job->get_user().get_gid());
   cache.Release();
+  Arc::Time processing_end;
+  Arc::Period processing_time = processing_end - processing_start;
+  if ((processing_time.GetPeriod() >= 1) || (processing_time.GetPeriodNanoseconds() > 100000000)) { // >0.1s
+    logger.msg(Arc::WARNING, "%s: Cache cleaning takes too long - %u.%06u seconds",
+                               job->get_id(), processing_time.GetPeriod(), processing_time.GetPeriodNanoseconds()/1000);
+  }
+
 }
 
 } // namespace ARex
