@@ -45,7 +45,7 @@ class GMJobQueueDTR: public GMJobQueue {
  */
 class DTRGenerator: public DataStaging::DTRCallback {
  private:
-  /** Active DTRs. Map of job id to DTR id. */
+  /** Active DTRs. Map of job id to DTR id(s). */
   std::multimap<std::string, std::string> active_dtrs;
   /** Jobs where all DTRs are finished. Map of job id to failure reason (empty if success)
      Finished jobs are stored only by ID because they references are already passed
@@ -81,7 +81,7 @@ class DTRGenerator: public DataStaging::DTRCallback {
   /** logger to a-rex log */
   static Arc::Logger logger;
   /** Central DTR LogDestination */
-  Arc::LogDestination* central_dtr_log;
+  DataStaging::DTRLogDestination central_dtr_log;
   /** Associated scheduler */
   DataStaging::Scheduler* scheduler;
 
@@ -164,7 +164,7 @@ class DTRGenerator: public DataStaging::DTRCallback {
    * sends them to the Scheduler.
    * @param job Job description object.
    */
-  void receiveJob(GMJobRef& job);
+  bool receiveJob(GMJobRef& job);
 
   /**
    * This method is used by A-REX to cancel on-going DTRs. A cancel request
