@@ -84,13 +84,13 @@ class AccountingControl(ComponentControl):
                     s_ids += s['GLUE2ServiceID']
 
             self.logger.debug('Running LDAP query over %s to find service endpoint URLs', args.top_bdii)
-            s_filter = reduce(lambda x, y: x + '(GLUE2EndpointServiceForeignKey={0})'.format(y), s_ids, '')
+            s_filter = reduce(lambda x, y: x + '(GLUE2EndpointServiceForeignKey={0})'.format(y.decode()), s_ids, '')
             endpoints = ldap_conn.search_st('o=glue', ldap.SCOPE_SUBTREE, attrlist=['GLUE2EndpointURL'], timeout=30,
                                             filterstr='(&(objectClass=Glue2Endpoint)(|{0}))'.format(s_filter))
             for (_, e) in endpoints:
                 if 'GLUE2EndpointURL' in e:
                     for url in e['GLUE2EndpointURL']:
-                        print(url.replace('stomp+ssl://', 'https://').replace('stomp://', 'http://'))
+                        print(url.decode().replace('stomp+ssl://', 'https://').replace('stomp://', 'http://'))
             ldap_conn.unbind()
         except ldap.LDAPError as err:
             self.logger.error('Failed to query Top-BDII %s. Error: %s.', args.top_bdii, err.message['desc'])
@@ -461,6 +461,7 @@ class AccountingControl(ComponentControl):
 
         accounting_actions = job_accounting_ctl.add_subparsers(title='Job Accounting Actions', dest='jobaction',
                                                                metavar='ACTION', help='DESCRIPTION')
+        accounting_actions.required = True
 
         accounting_job = accounting_actions.add_parser('info', help='Show job accounting data')
         accounting_job.add_argument('-o', '--output', default='all',
@@ -482,6 +483,7 @@ class AccountingControl(ComponentControl):
 
         accounting_actions = accounting_ctl.add_subparsers(title='Accounting Actions', dest='action',
                                                            metavar='ACTION', help='DESCRIPTION')
+        accounting_actions.required = True
 
         # add legacy accounting control as a sub-parser
         LegacyAccountingControl.register_parser(accounting_actions)
